/*

  iLogger
    
	Martin Matysiak, 2008
	    mail@k621.de

*/

#define F_CPU 4000000UL
#define LED_WARN PC1
#define LED_STAT PC0
#define LEDCODE_OFF 0x00
#define LEDCODE_OK 0x01
#define LEDCODE_WARNING 0x02
#define LEDCODE_PROCESSING 0x03

#include <avr/io.h>
#include <util/delay.h>
#include <gps.h>
#include <mmc.h>
#include <fat16.h>
 
struct {
	unsigned record:1;
	unsigned keydown:2;
} status;

int main (void) {
 
	//Ports definieren
	DDRC = 0x03;
	PORTC= LEDCODE_PROCESSING;

	//SD Karte + FAT initalisieren
	if ((mmc_init() != 0) || (InitFat16() != 0)) {
		PORTC = LEDCODE_WARNING;
		while(1) {
		}
	}

	PORTC = LEDCODE_OK;

	//Log-Datei suchen und Position merken
	File logFile;
	unsigned char * fileName = (unsigned char *)"log.csv";

	if (fexist_(fileName,&logFile) != 1) {
		PORTC = LEDCODE_WARNING;
		while(1) {
		}
	}

	//GPS Modul initalisieren, auf Fix warten
	gps_init();

	//Variablen initalisieren
	PORTC = LEDCODE_OFF;
	status.record = 0;

	//Hauptschleife
	while(1) {
		if (~PINC & (1 << PINC2)) {
			PORTC = LEDCODE_WARNING;
			if (!status.keydown) {
				status.keydown = 1;
				_delay_ms(2000);
			} else {
				PORTC = LEDCODE_PROCESSING;
				status.record = !status.record;
				if(status.record) {
					fopen_(fileName,'a',&logFile);
					fputs_(&logFile,"***START***");
				} else {
					fputs_(&logFile,"***STOP***");
					fclose_(&logFile);
				}
				_delay_ms(500);
				PORTC = LEDCODE_OFF;
			}
		} else {
			status.keydown = 0;
			PORTC = LEDCODE_OFF;
		}

		if (status.record) {
			fputchar_(&logFile, gps_get_char());
			PORTC ^= LEDCODE_OK;
		} else {
			_delay_ms(950);
			PORTC = LEDCODE_WARNING;
			_delay_ms(50);
			PORTC = LEDCODE_OFF;
		}
	}
 
	/* wird nie erreicht */
	return 0;
}
